# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"relu": "Activations.ipynb",
         "ReLU": "Activations.ipynb",
         "LeakyReLU": "Activations.ipynb",
         "leaky_relu": "Activations.ipynb",
         "Mish": "Activations.ipynb",
         "mish": "Activations.ipynb",
         "sigmoid": "Activations.ipynb",
         "softmax": "Activations.ipynb",
         "get_activation": "Activations.ipynb",
         "__activations__": "Activations.ipynb",
         "__inputDimError__": "Layers.ipynb",
         "Dense": "Layers.ipynb",
         "Conv2D": "Layers.ipynb",
         "MaxPool2D": "Layers.ipynb",
         "Flatten": "Layers.ipynb",
         "Activation": "Layers.ipynb",
         "Sequential": "Model.ipynb",
         "Data": "data.ipynb",
         "TestData": "data.ipynb",
         "TrainData": "data.ipynb",
         "create_db": "data.ipynb",
         "mse_loss": "losses.ipynb",
         "mae_loss": "losses.ipynb",
         "binary_cross_entropy": "losses.ipynb",
         "ce_loss": "losses.ipynb",
         "ce4softmax": "losses.ipynb"}

modules = ["activations.py",
           "layers.py",
           "model.py",
           "data.py",
           "losses.py"]

doc_url = "https://sachinruk.github.io/keraTorch/"

git_url = "https://github.com/sachinruk/keraTorch/tree/master/"

def custom_doc_links(name): return None
